#include <PalmOS.h>
#include "RedLedLib.h"


#define OUR_REQUIRED_RED_LED_LIB_VER		T3LedLibVer1000



Err redLedLibOpen(UInt16* ref){
	
	Err e;
	
	e=SysLibLoad(T3LedLibType,T3LedLibCreator,ref);
	
	return e;
}

void redLedLibClose(UInt16 ref){
	
	SysLibRemove(ref);
}

void testLed(char* color,UInt16 ref,UInt32 reservation){
	
	
	T3LedLibSetState(ref,reservation,MODE_ON,0,0);
	FrmCustomAlert(10024,color," LED should be on.",NULL);
	
	T3LedLibSetState(ref,reservation,MODE_OFF,0,0);
	FrmCustomAlert(10024,color," LED should be off.",NULL);
	
	T3LedLibSetState(ref,reservation,MODE_BLINK,50,100);
	FrmCustomAlert(10024,color," LED should be blinking: on for half a second every second.",NULL);
	
	T3LedLibSetState(ref,reservation,MODE_BLINK,1,50);
	FrmCustomAlert(10024,color," LED should be blinking: on for 1/100 second every half second.",NULL);
	
	T3LedLibSetState(ref,reservation,MODE_BLINK,1,10);
	FrmCustomAlert(10024,color," LED should be blinking: on for 1/100 second every 1/10 second.",NULL);
	
	T3LedLibSetState(ref,reservation,MODE_OFF,0,0);
	FrmCustomAlert(10024,color," LED should be off.",NULL);
	
}

void reserveAndTest(UInt16 ref,UInt32 led,char* color){
	
	UInt32 reservation;
	
	
	reservation=T3LedReserve(ref,led);
	
	if(reservation==0){
		
		ErrAlertCustom(0,color, " LED reservation failed!",NULL);
		return;
		
	}
	if(T3LedReserve(ref,led)!=0){		//make sure we cannot reserve this twice (this is not needed, i have it here to see
										//if library handles this as needed
		
		
		SysFatalAlert("Led was allowed to be reserved twice. this should never happen. Library error!");
		
	}
	
	testLed(color,ref,reservation);
	
	T3LedRelease(ref,reservation);
}

void coolEffect(UInt16 ref){
	
	UInt32 red_reservation;
	UInt32 grn_reservation;
	
	red_reservation=T3LedReserve(ref,LED_RED);
	grn_reservation=T3LedReserve(ref,LED_GREEN);
	
	
	if(red_reservation && grn_reservation){	//if we managed to get them both reserved
		
		T3LedLibSetState(ref,red_reservation,MODE_BLINK,20,40);	//set leds to blink in different speeds to make a cool effect
		T3LedLibSetState(ref,grn_reservation,MODE_BLINK,25,50);
		
		FrmCustomAlert(10024,"Check this out!",NULL,NULL);
		
		T3LedLibSetState(ref,red_reservation,MODE_OFF,0,0);	//turn them off
		T3LedLibSetState(ref,grn_reservation,MODE_OFF,0,0);
	
	}
	else{
		
		ErrAlertCustom(10024,"coolEffect(): failed to reserve both leds at once",NULL,NULL);
		
	}
	
	if(red_reservation){		//if red is reserved, release it
		
		T3LedRelease(ref,red_reservation);
		
	}
	
	if(grn_reservation){		//if green was reserved, release it
		
		T3LedRelease(ref,grn_reservation);
		
	}
	
}

void redLedLibTestMain(UInt16 ref){		//lib is loaded, do the testing here
	
	Err e;
	
	reserveAndTest(ref,LED_RED,"Red");
	reserveAndTest(ref,LED_GREEN,"Green");
	
	coolEffect(ref);
	
}

void redLedLibTest(){
	
	UInt16 ref;
	Err e;
	
	
	//load the library
	
	e=redLedLibOpen(&ref);
	
	if(e){
		
		ErrAlertCustom(e,"Error occured while opening the library: ",NULL,NULL);
		return;
		
	}
	else{
		
		FrmCustomAlert(10024,"Library ","opened"," ok");
		
	}
	
	//check version
	
	if(T3LedLibGetVersion(ref)<OUR_REQUIRED_RED_LED_LIB_VER){
		
		char x[9];
		
		StrIToH(x,T3LedLibGetVersion(ref));
		
		
		ErrAlertCustom(0,"too old, got ver=0x","Library ",x);
		
	}
	else
	{	//this version is ok
		
		FrmCustomAlert(10024,"Library ","version"," ok");
		
		redLedLibTestMain(ref);
		
	}
	
	redLedLibClose(ref);
}




UInt32 PilotMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
	
	if(cmd==0){
		
		redLedLibTest();
		
		
	}
	
	return errNone;
}
